pro Set_Dither, Event, struct
;
;	determines the sky dither position for the images in a structure
;
;	for use in Step 6A
;
;	 based on locating either larger or smaller time jump steps
;		between shorter or longer exposures
;
;	DPC		20090922
;			20091018 	mods
;			20091228	minor bug fix
;
;
;---------------------------------------
debug = 0
;
;	recover time offsets (in min)
;
time = struct.julian_date
file_name = struct.file_name
;
;	parse file names into frame numbers
;
first = STRPOS(file_name, '.')
last = STRPOS(file_name, '_')
length = last - first - 1
;
n_file = N_ELEMENTS(file_name)
for i_file = 0, n_file-1 do begin
	;
	file_name[i_file] = STRMID(file_name[i_file], first[i_file] + 1, length[i_file])
endfor
file_number = LONG(file_name)
;
;	and next nearest offsets
;
n_t = N_ELEMENTS(time)
;
time_1 = time[0:n_t-2]
time_2 = time[1:n_t-1]
dt = time_2 - time_1
;
file_num_1 = file_number[0:n_t-2]
file_num_2 = file_number[1:n_t-1]
dn = file_num_2 - file_num_1
;
;	select which dt's to use, based on requiring dn=1
;
ind_dn1 = WHERE(dn eq 1, n_dn1)
;
;	use MFM to find the dominant difference
;
zz = median_filtered_mean(dt[ind_dn1])
;
;print, "for ",n_t," time steps, the mean dt = ",zz[0],"+/-",zz[1]
;
;	grab all the others (with values > 0.25 min) and mfm them to reject outliers
;
if(zz[0] gt 0.15) then dt_test = 0.225
if(zz[0] le 0.15) then dt_test = 0.14
if(zz[0] lt 0.06) then dt_test = 0.10 
;
if (debug) then print,"found ",n_dn1," close dt values with dn=1. mean, sigma = ",zz[0],"+/-",zz[1]
;
ind_others = WHERE(abs(dt - zz[0])/zz[1] gt 4.0 and dt gt dt_test ,n_other)
if(n_other gt 0) then begin
	zz_other = median_filtered_mean(dt[ind_others])
	;
if(debug) then	print,"found ",n_other," different dt values with mean = ",zz_other[0],"+/-",zz_other[1]
	;
	;	at this point, we have the mean dt and dt_other, so proceed to classify
	;
endif else begin
	;
	;	failed to find differences - call this out and set fail conditions
	;
	struct[*].dither = -99
	;
	stop
	;
	goto, the_end
endelse
;
;	to be sure we are classifying dither positions, start at 1 (ie. skip 0)
;
this_dither = 1
;
;	first image is part of first dither, by def
;
struct[0].dither = this_dither
;
for i_dt = 0, n_t-2 do begin
	;
	; is this dt closer to "normal" or "other"?
	;
	test1 = abs(dt[i_dt] - zz[0])/(zz[1] > 0.01)
	test2 = abs(dt[i_dt] - zz_other[0])/(zz_other[1] > 0.01)
	;
	if(test1 lt test2) then begin
		;
		;	closer to "normal"
		;
		struct[i_dt+1].dither = this_dither
		;
	endif else begin
		;
		;	closer to "other", bump dither number, if some key conditions are
		;	present:
		;
		;		1. very long time gap (9 sigma away from normal) *and* file_number jump is 1 (not more)
		;		-or-
		;		2. the HWP angle is zero
		;
		;		if *Neither* are true, don't change dither number
		;
		if(test1 gt 8.0 and dn[i_dt] eq 1) then begin
			this_dither++
		endif else begin
			if(struct[i_dt+1].hwp eq 0.0) then begin
				;
				;	test for next image being hwp=0 also
				;
				if(struct[i_dt].hwp ne struct[i_dt+1].hwp and struct[i_dt].hwp ne 0.0) then begin
					if(debug) then begin
						print,"bumping up dither number to ",this_dither+1," for i_dt = ",i_dt
						print,"test1, test2, dn[i_dt] = ",test1, test2, dn[i_dt]
						print,"hwp[i_dt], hwp[i_dt+1] = ",struct[i_dt].hwp, struct[i_dt+1].hwp
					endif
					;
					this_dither++
				endif
			endif
		endelse
		struct[i_dt+1].dither = this_dither
	endelse
	;
	if(debug) then print, "i_dt, dt, test1, test2, dither = ",i_dt, dt[i_dt], test1, test2, this_dither
	;
endfor
;
;	test - how many dither positions did we get? If > 6, problems unless combined pointings
;
if(this_dither gt 6) then begin
	print_text, Event, "WARNING - found too many dither positions "+STRING(this_dither,format='(i2)')
endif else begin
	if(this_dither lt 6) then begin
		print_text, Event, "WARNING - found too few dither positions "+STRING(this_dither,format='(i2)')
	endif
endelse
;
the_end:
end
