;	dpc_sky_combine_sub is second function below
;
function DPC_MEDIAN_BOX, image, in_kernel;
;
;	convolves image with box-in-a-box kernel, with
;	inner radius = kernel/2 (inside of which there is
;	no sensitivity) and outer radius = kernel
;	(with full sensitivity) -- this to avoid stomping
;	on PSF wings...
;
;------------------------------------------------
;
;	fill box with sensitivity values
;
kernel = ROUND(in_kernel)
kernel_sq = kernel * kernel
k4 = ROUND((kernel/2)^2)
;
out_image = image * 0.0
rad_image = fltarr(2*kernel+1,2*kernel+1)
;
i_size = SIZE(image)
nx = i_size[1]
ny = i_size[2]
;
for ix = 0, nx-1 do begin
	xmin = 0 > (ix-kernel)
	xmax = (nx-1) < (ix+kernel)
	;
	for iy = 0, ny-1 do begin
		ymin = 0 > (iy-kernel)
		ymax = (ny-1) < (iy+kernel)
		;
		sub_image = image[xmin:xmax,ymin:ymax]
		;
		for jx = 0, xmax-xmin do begin
			for jy = 0, ymax-ymin do begin
				rad_image[jx,jy] = (ix-jx-xmin)^2 + (iy-jy-ymin)^2
			endfor
		endfor
		;
		ind = WHERE(rad_image[0:xmax-xmin,0:ymax-ymin] le kernel_sq and rad_image[0:xmax-xmin,0:ymax-ymin] ge k4,count)
		if(count gt 1) then begin
			vec = REFORM(sub_image[ind])
			zz = median_filtered_mean(vec)
			out_image[ix,iy] = zz[0]
		endif else begin
			out_image[ix,iy] = 0.0
		endelse
	endfor
endfor
;
return, out_image
end


function dpc_sky_combine_sub, data_cube, image_quality, kernel_size
;
;   subroutine version of image combiner, ala IRAF imcombine - for calling by tp_image_combine.pro
;
;   returns the mean sky image, scaled to 100.0
;
;   accepts list of images and performs median-filtered-mean
;   averaging, producing an averaged image
;
;   this version first traps high, low values then cleans all deleted pixels
;   before matching sky means and then running median filter to remove all stars
;
;   DPC 20060102    copied from imcombine and modified for tp application
;       20060103     cut down to subroutine
;       20060107     modified to input data cube, not name list
;       20060124     increased size of initial median kernel to 32 from 16,
;                      added final median filtering of combined image, made
;                      kernel size a parameter (maybe for passing later?)
;       20060803 added switch for selection of median or original data
;                      (K-band may be better with original)
;------------------------------------------------------------------------------
;
select_switch = 0     ; 0=median; 1=no median
;
deleted_data = -1.0d6
;
;   median filter kernel size (in pixels):
;
kernel = kernel_size
;
cube_size = SIZE(data_cube)
nimages = cube_size[1]
;
ncol = cube_size[2]
nrow = cube_size[3]
;
xsize = ncol
ysize = nrow
;
;   reserve this much space for the output images
;
image_mean = dblarr(xsize, ysize)
;
;   and the monster array to hold all the images
;
big_image = data_cube
;
;   read in all the images into the big_image array
;
for ifile = 0, nimages-1 do begin
    ;
    ;  but skip over bad ones
    ;
    if(image_quality[ifile] eq 1) then begin
        ;
        ;   find mean sky level, then scale to 100
        ;
        ;   first, do a median filter on each image
        ;
        if(kernel gt 1) then begin
            ;raw_image = DPC_MEDIAN_BOX(REFORM(data_cube[ifile,*,*]), kernel)
            raw_image = MEDIAN(REFORM(data_cube[ifile,*,*]), kernel)
        endif else begin
            raw_image = REFORM(data_cube[ifile,*,*])
        endelse
        ;
        pol_find_sky, raw_image, sky_mean, sky_sigma
        ;
        scale = 100.0/sky_mean
        ;
        ;   now, scale either the median or the original image
        ;   based on this mean value
        ;
        if(select_switch eq 0) then begin
            raw_image = raw_image * scale
            ;
            ;   pack constituent image into big_image
            ;
            big_image[ifile,*,*] = raw_image[*,*]
        endif else begin
            big_image[ifile,*,*] = big_image[ifile,*,*] * scale
        endelse
    endif
endfor
;
;   for every pixel in the final resulting images...
;
ind_good = WHERE(image_quality eq 1)
for icol = 0, xsize-1 do begin
    for irow = 0, ysize-1 do begin
        ;
        ;     load the good data into a vector for median-filtering
        ;
        vector = dblarr(nimages)
        ;
        ;   don't include zeroed pixels -- these indicate large region deleted (e.g., guider shadow)
        ;
        vector = REFORM(big_image[*,icol,irow])
        vector = vector[ind_good]  ; just the good images
        ;
        ind = WHERE(vector ne 0.0,ncount)
        if(ncount ne 0) then begin
            clean_vector = vector[ind]
            ;
            ;     compute median-filtered-mean and sigma
            ;
            ;mx = median_filtered_mean(clean_vector)     ; [0] is mean; [1] is sigma
            ;
            ;   use MEDIAN function instead
            ;
            mx = MEDIAN(clean_vector)
            ;
            image_mean[icol,irow] = mx
        endif else image_mean[icol,irow] = 0.     ; all in stack were zero
    endfor
endfor
;
;   do a final MEDIAN filter over the resulting image, if select switch reset
;
if(select_switch eq 0 and kernel gt 1) then image_mean = MEDIAN(image_mean, kernel)
;
return, image_mean
end

