function g_make_PSF, ng, xarb, yarb, PSF_model, PSF_gaussian, PSF_rad, PSF_mag, sum_resid_f, status
;
;	creates a new PSF appropriate for the image location (xarb, yarb)
;	as based on the PSF_model functions.
;
;	returns the array of residuals
;
;	also returns the PSF gaussian parameters via PSF_gaussian
;	and the "psfmag" via PSF_mag
;
;	DPC	20080721
;		20081227	added status variable (0=OK, -1=fail)
;
;---------------------------------------------------------------------
;
;	PSF model functions are:
;
;		A. 2nd order with coefficients in the SFIT/MAX_DEGREE order:
;				k, y, y2, x, xy, x2
;		B. 1st order with coeffs
;				k, y, x
;		C. 0th order, with no coeffs
;
;---------------------------------------------------------------------
;
;	PSF_model structure is :
;
;gauss_fit = dblarr(5,0 or 3 or 6)					; gaussian term x SFIT kx term
;resid_fit = dblarr(nxypsf, nxypsf,0 or 3 or 6)
;;
;PSF_model = {	nxypsf: nxypsf, $
;				psfmag: psfmag, $
;				rad_psf: rad_psf, $
;				fit_rad: fit_rad, $
;				gauss_fit: gauss_fit, $
;				resid_fit: resid_fit $
;			 }
;
;----------------------------------------------------------------------
;
;	begin by computing the gaussian parameters
;
nxypsf = PSF_model.nxypsf
PSF_rad = double(PSF_model.rad_psf)
fit_rad = double(PSF_model.fit_rad)
x = double(xarb)
y = double(yarb)
;
xcen = float(nxypsf/2.)-0.5
ycen = xcen
;
resid = dblarr(nxypsf, nxypsf)
;
PSF_gaussian = dblarr(5)
;
for ig = 0, 4 do begin
	case ng of
		1: begin
			PSF_gaussian[ig] = PSF_model.gauss_fit[ig]
		   end
		2: begin
			PSF_gaussian[ig] = PSF_model.gauss_fit[ig,0] + PSF_model.gauss_fit[ig,1] * y $
					   + PSF_model.gauss_fit[ig,2] * x
		   end
		3: begin
			PSF_gaussian[ig] = PSF_model.gauss_fit[ig,0] + PSF_model.gauss_fit[ig,1] * y $
					   + PSF_model.gauss_fit[ig,2] * x
		   end
		else: begin
			PSF_gaussian[ig] = PSF_model.gauss_fit[ig,0] + PSF_model.gauss_fit[ig,1] * y $
					   + PSF_model.gauss_fit[ig,2] * (y * y) + PSF_model.gauss_fit[ig,3] * x $
					   + PSF_model.gauss_fit[ig,4] * x * y + PSF_model.gauss_fit[ig,5] * x * x
		   end
	endcase
endfor
;
;print, "PSF_gaussian fitted (inside g_make_psf) = ",PSF_gaussian
;
;	and all the residuals (and counts within psf_rad)
;
sum_resid_f = 0.0d0
;sum_resid_p = 0.0d0
for ix = 0, nxypsf-1 do begin
	for iy = 0, nxypsf-1 do begin
		case ng of
			1: begin
				resid[ix,iy] = PSF_model.resid_fit[ix,iy]
			   end
			2: begin
				 resid[ix,iy] = PSF_model.resid_fit[ix,iy,0] + PSF_model.resid_fit[ix,iy,1] * y $
					   + PSF_model.resid_fit[ix,iy,2] * x
			   end
			3: begin
				 resid[ix,iy] = PSF_model.resid_fit[ix,iy,0] + PSF_model.resid_fit[ix,iy,1] * y $
					   + PSF_model.resid_fit[ix,iy,2] * x
			   end
			else: begin
					resid[ix,iy] = PSF_model.resid_fit[ix,iy,0] + PSF_model.resid_fit[ix,iy,1] * y $
					   + PSF_model.resid_fit[ix,iy,2] * (y * y) + PSF_model.resid_fit[ix,iy,3] * x $
					   + PSF_model.resid_fit[ix,iy,4] * x * y + PSF_model.resid_fit[ix,iy,5] * x * x
                  end
        endcase
		dx = float(ix) - xcen
		dy = float(iy) - ycen
		rad = sqrt(dx * dx + dy * dy)/2.
		if(rad le fit_rad) then begin
			sum_resid_f = sum_resid_f + 0.25 * resid[ix,iy]
;			sum_resid_p = sum_resid_p + 0.25 * resid[ix,iy]
		endif else begin
;			if(rad le PSF_rad) then begin
;				sum_resid_p = sum_resid_p + 0.25 * resid[ix,iy]
;			endif
		endelse
	endfor
endfor
;
;	next, compute psf mag from gaussian sum and resid sum
;
gauss_sum = PSF_gaussian[0] * !DPI * 2.0 * abs(PSF_gaussian[3] * PSF_gaussian[4])
;
total_counts = sum_resid_f + gauss_sum
;
;PSF_mag = 25.0 - 2.5d0 * alog10 (total_counts)			;
;;
;if(abs(PSF_mag - 15.0) gt 0.01) then begin
;	print, 'G_MAKE_PSF: PSF_mag = ',PSF_mag,' instead of 15.0 - rescaling'
;	scale = 10^(-0.4d0*(15.0 - PSF_mag))
;	PSF_gaussian[0] = PSF_gaussian[0] * scale
;	resid = resid * scale
;	PSF_mag = 15.0
;	sum_resid_f = sum_resid_f * scale
;endif
;
;PSF_mag = 15.000		; just set it
;
PSF_mag = PSF_model.psfmag
;
;print, "G_MAKE_PSF: calculated PSFmag = ",PSF_mag
;
if(NOT(FINITE(PSF_mag))) then begin
	print, "PSFMAG not finite in g_make_psf. Blowing up on purpose..."
	status = -1
	return, resid
endif
;
status=0
return, resid
;
end