pro S7_fit_using_S11_points, Event, input_image, fit_option_mode, s11_ptr, mean_full_sky, $
									mean_full_sigma, x_off, y_off, file_path
;
;	fits the input_image using one of the three fit_modes,
;	based on the S11-type points and keywords on heap
;	and pointed to by S11_ptr
;
;	DPC 20120408 added inclusion of offsets from coadding to register with S11 coadded image used
;				 to generate the points
;
;					X_off is the offset of the LL corner of this image to the overall coadded image
;							such that no point with X value less than X_off is on this image and
;							all X values need to have X_off subtracted to get onto the X coord for
;							this image. (all same for Y_off)
;
debug = 0
;
isize = SIZE( input_image )
ncol = isize[1]
ncol2 = FLOOR(ncol / 2)
nrow = isize[2]
nrow2 = FLOOR(nrow / 2)
;
npts = (*s11_ptr).npts
radius = (*s11_ptr).radius
order = (*s11_ptr).order
;
;	if debug set, open a reg file for writing
;
if(debug eq 1) then begin
	reg_file = FILE_DIRNAME(file_path, /MARK_DIRECTORY) + FILE_BASENAME(file_path,'.fits')+'_s7_reg.reg'
	openw, lu_reg, reg_file, /GET_LUN
endif
;
;	use mode to determine how many loops and loop ends
;
case fit_option_mode of
	0: begin
		;
		;	all 4 Quads together, so one zone
		;
		mx = 1
		my = 1
		x0 = [0]
		x1 = [ncol-1]
		y0 = [0]
		y1 = [nrow-1]
	   end
	1: begin
		;
		;	all 4 Quads independent, so 4 Zones
		;
		mx = 2
		my = 2
		x0 = [0, ncol2]	; LL, UL, LR, UR
		x1 = [ncol2-1, ncol-1]
		y0 = [0, nrow2]
		y1 = [nrow2-1, nrow-1]
	   end
	2: begin
		;
		;	top and bottom independently
		;
		mx = 1
		my = 2
		x0 = [0, 0]
		x1 = [ncol-1, ncol-1]
		y0 = [0, nrow2]
		y1 = [nrow2-1, nrow-1]
	   end
endcase
;
;	create the model image (from original, times zero)
;
ptr_model = PTR_NEW( input_image )
;
;	for the indicated number of zones
;
zone_sky = fltarr(mx,my)
zone_sigma = fltarr(mx,my)
;
for jx = 0, mx-1 do begin
;	print, 'starting x loop with jx = ',jx
	this_x0 = x0[jx]
	this_x1 = x1[jx]
;	print, 'this_x0, _x1 = ',this_x0, this_x1
	;
	for jy = 0, my-1 do begin
;		print, 'starting y loop with jy = ',jy
		this_y0 = y0[jy]
		this_y1 = y1[jy]
;		print, 'this y0, _y1 = ',this_y0, this_y1
		;
		;	fit background
		;
		;		for each point, compute mfm value of the zone
		;
		mean_xy = fltarr(npts,3)*0.0
		rms_xy = fltarr(npts)*0.0
		nmean=0L
		;
		for i = 0, npts-1 do begin
			;
			mean_xy[nmean,*] = 0.0
			;
			cx = (*s11_ptr).xy_points[i,0]
			cy = (*s11_ptr).xy_points[i,1]
			;
			;	correct for offsets and limits
			;
			cx -= x_off
			cy -= y_off
			;
			;	compare to current zone - if point center in zone, keep
			;
			if((cx le this_x1) and (cx ge this_x0) and $
				(cy le this_y1) and (cy ge this_y0)) then begin
				;
				;	if debug set, mark these centers on reg file
				;
				if(debug eq 1) then begin
					printf, lu_reg, cx, cy, radius, format='("circle ",f7.1,1x,f7.1,1x,I2,1x, "# color= white width=  1")'
				endif
				;
				;	collect all the image pixels within radius of this point
				;
				xr0 = this_x0 > (cx - radius)
				xr1 = (cx + radius) < this_x1
				yr0 = this_y0 > (cy - radius)
				yr1 = (cy + radius) < this_y1
				;
				vec = fltarr( (xr1 - xr0 + 1) * (yr1 - yr0 + 1) )
				nvec = 0L
				;
				for ix = xr0, xr1 do begin
					for iy = yr0, yr1 do begin
						;
						dx = float(ix - cx)
						dy = float(iy - cy)
						dr = sqrt(dx * dx + dy * dy)
						if(dr le radius) then begin
							vec[nvec] = input_image[ix,iy]
							;
							mean_xy[nmean,0] += float(ix)
							mean_xy[nmean,1] += float(iy)
							nvec++
						endif
					endfor
				endfor
				;
				mean_xy[nmean,0] /= float(nvec)
				mean_xy[nmean,1] /= float(nvec)
				;
				vec = vec[0:nvec-1]
				ind = WHERE(FINITE(vec),nfinite)
				;
				if(nfinite gt 0) then begin
					zz = median_filtered_mean(vec[ind])
					if(FINITE(zz[0]) and FINITE(zz[1])) then begin
						mean_xy[nmean,2] = zz[0]
						rms_xy[nmean] = zz[1]
						nmean++
					endif
				endif
			endif
		endfor
		;
		;
;		print, 'for this x,y zone, nmean = ',nmean
		;
		;
		;	mean sky and mean rms
		;
		zz_sky = median_filtered_mean(REFORM( mean_xy[0:nmean-1,2] ))
		zz_rms = median_filtered_mean(rms_xy[0:nmean-1])
		;
		mean_sky = zz_sky[0]
		sky_noise = zz_rms[0]
		;
		zone_sky[jx,jy] = mean_sky
		zone_sigma[jx,jy] = sky_noise
		;
		;	clean out baddies
		;
		test_xy = REFORM(mean_xy[0:nmean-1,2])
		;
		ind_good = WHERE( abs(test_xy - mean_sky)/sky_noise lt 10.0,ngood)
		;
;		print, '   and ngood = ',ngood
		;
		if(ngood ge 0) then begin
			mean_xy = mean_xy[ind_good,*]
		endif else begin
			print_text, Event, 'Too few good points for background fit'
			(*ptr_model)[this_x0:this_x1,this_y0:this_y1] = 0.0
			goto, y_loop_end
		endelse
		;
		;	have all the points. Next, select maximum fitting order
		;	supported by the data
		;
		order_i = order
		;
		order_loop:
		min_pts = CEIL(float(order_i) * (float(order_i - 1 ) * 2.5))
		if(ngood lt min_pts) then begin
			order_i--
			if(order_i le 0) then stop
			goto, order_loop
		endif
		;
		degree = order_i < ngood
		;
		Result = SFIT( TRANSPOSE( mean_xy), degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
		;
		this_y = findgen(this_y1 - this_y0 + 1)+0.5+this_y0
		;
		for ix = this_x0, this_x1 do begin
			this_x = float(ix)+0.5
			;
			(*ptr_model)[ix,this_y0:this_y1] = 0.0
			;
			ik = 0
			for kx = 0, degree do begin
				for ky = 0, degree do begin
					if(kx+ky le degree) then begin
						(*ptr_model)[ix,this_y0:this_y1] += coeffs[ik] * (this_y^ky) * (this_x^kx)
						ik++
					endif
				endfor
			endfor
		endfor
		;
		;	save the difference image
		;
		(*ptr_model)[this_x0:this_x1,this_y0:this_y1] *= -1.0
		(*ptr_model)[this_x0:this_x1,this_y0:this_y1] += input_image[this_x0:this_x1,this_y0:this_y1]
		;
		;	next Y zone
		;
		print, 'S7 fit with S11 points for image ',FILE_BASENAME(file_path),' quad mode = ',fit_option_mode,$
		 	' quad being fit (jx,jy) = ',jx,jy,' degree = ',degree,' points = ',ngood
		;
		y_loop_end:
	endfor
	;
	;	next X zone
	;
endfor
;
;	compute the mean background and sigma
;
vec = REFORM( zone_sky[*,*] )
zz_sky = median_filtered_mean(vec)
vec_sig = REFORM( zone_sigma[*,*] )
zz_sig = median_filtered_mean(vec_sig)
;
mean_full_sky = zz_sky[0]
mean_full_sigma = zz_sig[0]
;
;	save result in input image and return
;
input_image = (*ptr_model)
input_image += zz_sky[0]					; put mean sky back into image
;
if(debug eq 1) then begin
	FREE_LUN, lu_reg
endif
;
;	free heap
;
PTR_FREE, ptr_model
;
end